import { PropBindType } from "@cloudbase/cals"
import { autorun, observable, remove, untracked } from "mobx"
import { EventProxy } from './eventProxy';
import { generateDataBinds } from "./adapter/generate"
import { debounce } from "lodash-es";

/**
 * 以 widget id 为 key，只记录同个 widget 的最后一次表达式的结果
 * 这里因为同时只会执行（调试）一个表达式，所以只需保存最后一次执行的结果即可
 */

const eventProxy = new EventProxy()

function debounceFnFactory(fn: Function, wait: number, ...args) {
  const id = `${fn.name}:${JSON.stringify(args)}`;
  debounceFnFactory[id] = debounceFnFactory[id] || debounce(fn.bind(this, ...args), wait);
  return debounceFnFactory[id];
}

export let debugExprMap = observable<{
  node: {
    [nodeId: string]: {
      expr: string,
      result: any,
    }
  }
}>({node: {}})

export function resetDebugExprMap() {
  // FIXME: widgets 中 for 之下 widget 的 dispose 没有被触发，所以这边通过 remove 来删除使原来的 autorun 不再触发
  remove(debugExprMap, 'node')
  debugExprMap.node = {}
  // window['__debugExprMap'] = debugExprMap
}

export function autorunForDebugExpr({
  widgetId,
  subForItems,
  globalCtx,
  getDataContextFn,
  get$wFn,
  get$scopeFn,
}) {
  const forDebugExprDispose = autorun(() => {
    // FIXME: 暂时没考虑低码组件开发模式
    const dataContext = untracked(getDataContextFn);
    const $w = untracked(get$wFn);
    const $scope = untracked(get$scopeFn);

    untracked(() => {
      if (!debugExprMap.node[widgetId]) {
        debugExprMap.node[widgetId] = {
          expr: '',
          result: ''
        }
      }
    })

    const expr = debugExprMap.node[widgetId].expr
    untracked(() => {
      debugExprMap.node[widgetId].result = '' // 先清空，这样如果下边失败了也不会有错误的值
      const fn = generateDataBinds(globalCtx, [
        {
          type: PropBindType.expression,
          bindDataPath: expr,
          propertyPath: 'expr',
        },
      ])?.['expr']
      if (fn) {
        const result = fn(subForItems, undefined, dataContext, $w, $scope)
        debugExprMap.node[widgetId].result = result
      }

      // 这样对于数组只会拿最后一项
      debounceFnFactory((widgetId: string) => eventProxy.emit('done_' + widgetId), 300, widgetId)()

    })
  })
  return forDebugExprDispose
}

export function evalExpr(widgetId: string, expr: string) {
  return new Promise(resolve => {
    if (!debugExprMap.node[widgetId]) resolve(undefined)

    if (debugExprMap.node[widgetId].expr === expr) {
      // 表达式没变，直接返回结果
      resolve(debugExprMap.node[widgetId].result)
    } else {
      eventProxy.once('done_' + widgetId, () => {
        resolve(debugExprMap.node[widgetId].result)
      })

      // 触发 autorun
      debugExprMap.node[widgetId].expr = expr
    }
  })
}

// window['__testExpr'] = evalExpr





